import { useCallback, useEffect, useState } from "preact/hooks";
import { DataRow } from "../api/DataRow";
import style from './index.module.scss';
import { Posts } from "../components/Posts";

import InfiniteScroller from 'react-infinite-scroller';
import { DraggablePostsList } from "../components/DraggablePostsList";
import { DataRowSelection } from "../api/DataRowSelection";

export function Index() {
    const [ data, setData ] = useState(null as null | DataRow[]);
    const [ selected, setSelected ] = useState(null as null | DataRow[]);
    const dataRowSelection = new DataRowSelection(selected ?? []);

    const [ hasMore, setHasMore ] = useState(true);
    const [ fetching, setFetching ] = useState(false);

    useEffect(() => {
        (async () => {
            setSelected(await dataRowSelection.get());
        })();
    }, []);

    const fetchMore = useCallback(
        async function(_page: number) {
            if (fetching) return;

            setFetching(true);
            // await new Promise(r => setTimeout(r, 1000));
            const newData = await DataRow.find({
                page: data ? Math.floor(data.length / 20) : 0
            });
            setData([ ...data ?? [], ...newData ]);
            setHasMore(newData.length === 20);
            setFetching(false);
        },
        [ fetching, hasMore ]
    );

    async function saveSelection(rows: DataRow[]) {
        setSelected(rows);
        dataRowSelection.set(rows);
        await dataRowSelection.save();
    }

    function select(row: DataRow) {
        console.log(row);
        if (data) {
            if (data.indexOf(row) == -1) throw new Error('Selected element that does not exist');
            saveSelection([ ...selected ?? [], row ])
            setData(data)
        }
    }

    return (
        <div className={style.index}>
            
            {
                selected
                    ?
                        <div className={style.selection}>
                            <h2>Selection</h2>
                            <DraggablePostsList rows={selected} setRows={saveSelection} />
                        </div>
                    : null
            }
            
            <InfiniteScroller
                pageStart={0}
                loadMore={fetchMore}
                hasMore={hasMore}
                loader={<pre>loading . . .</pre>}
                useWindow={true}
                >
                {
                    data
                        ? <Posts rows={data.filter(x => selected ? selected?.indexOf(x) === -1 : true)} onSelectChange={select} />
                        : <></>
                }
            </InfiniteScroller>
        </div>
    )
}